import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../../domain/entities/secure_storage_keys.dart';

abstract class SecureStorageDatasource {
  Future<void> deleteUser({required String username});
  Future<void> deleteToken({required String username});
  Future<void> deleteTokenExpirationDate({required String username});
  Future<UserModel?> readUser({required String username});
  Future<TokenModel?> readToken({required String username});
  Future<DateTime?> readTokenExpirationDate({required String username});
  Future<String?> readLastUser();
  Future<void> writeUser(UserModel user);
  Future<void> writeToken(TokenModel token);
  Future<void> writeTokenExpirationDate(
      String username, String tenantDomain, DateTime tokenExpirationDate);
  Future<void> writeLastUser(
    String username,
    String tenantDomain,
  );
  Future<void> deleteLastUser();
}

class SecureStorageLocalDatasource implements SecureStorageDatasource {
  late final FlutterSecureStorage _storage;

  SecureStorageLocalDatasource({FlutterSecureStorage? storage}) {
    _storage = storage ?? const FlutterSecureStorage();
  }

  @override
  Future<UserModel?> readUser({required String username}) async {
    final jsonStr =
        await _storage.read(key: '$username-${SecureStorageKeys.user}');

    if (jsonStr != null) {
      return UserModel.fromJson(jsonStr);
    }

    return null;
  }

  @override
  Future<TokenModel?> readToken({required String username}) async {
    final jsonStr =
        await _storage.read(key: '$username-${SecureStorageKeys.token}');

    if (jsonStr != null) {
      return TokenModel.fromJson(jsonStr);
    }

    return null;
  }

  @override
  Future<DateTime?> readTokenExpirationDate({required String username}) async {
    final jsonStr = await _storage.read(
        key: '$username-${SecureStorageKeys.tokenExpirationDate}');

    if (jsonStr != null) {
      return DateTime.tryParse(jsonStr);
    }

    return null;
  }

  @override
  Future<void> writeUser(UserModel user) async {
    String keyUser = user.username;
    if (!keyUser.contains('@')) {
      keyUser += '@${user.tenantDomain}';
    }
    await _storage.write(
        key: '$keyUser-${SecureStorageKeys.user}', value: user.toJson());
  }

  @override
  Future<void> writeToken(TokenModel token) async {
    await _storage.write(
        key: '${token.username}-${SecureStorageKeys.token}',
        value: token.toJson());
  }

  @override
  Future<void> writeTokenExpirationDate(String username, String tenantDomain,
      DateTime tokenExpirationDate) async {
    String keyUser = username;
    if (!keyUser.contains('@')) {
      keyUser += '@$tenantDomain';
    }
    await _storage.write(
        key: '$keyUser-${SecureStorageKeys.tokenExpirationDate}',
        value: tokenExpirationDate.toIso8601String());
  }

  @override
  Future<void> deleteUser({required String username}) async {
    await _storage.delete(key: '$username-${SecureStorageKeys.user}');
  }

  @override
  Future<void> deleteToken({required String username}) async {
    await _storage.delete(key: '$username-${SecureStorageKeys.token}');
  }

  @override
  Future<void> deleteTokenExpirationDate({required String username}) async {
    await _storage.delete(
        key: '$username-${SecureStorageKeys.tokenExpirationDate}');
  }

  @override
  Future<String?> readLastUser() async {
    final jsonStr = await _storage.read(key: SecureStorageKeys.lastUser);

    if (jsonStr != null) {
      return jsonStr;
    }

    return null;
  }

  @override
  Future<void> writeLastUser(String username, String tenantDomain) async {
    String keyUser = username;
    if (!keyUser.contains('@') && tenantDomain.isNotEmpty) {
      keyUser += '@$tenantDomain';
    }
    await _storage.write(key: SecureStorageKeys.lastUser, value: keyUser);
  }

  @override
  Future<void> deleteLastUser() async {
    await _storage.delete(key: SecureStorageKeys.lastUser);
  }
}
