import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../datasources/secure_storage_local_datasource.dart';

abstract class SecureStorageRepository {
  Future<void> deleteUser({required String username});
  Future<void> deleteToken({required String username});
  Future<void> deleteTokenExpirationDate({required String username});
  Future<void> deleteLastUser();
  Future<User?> readUser({required String username});
  Future<Token?> readToken({required String username});
  Future<DateTime?> readTokenExpirationDate({required String username});
  Future<String?> readLastUser();
  Future<void> writeUser(User user);
  Future<void> writeToken(Token user);
  Future<void> writeTokenExpirationDate(
      String username, String tenantDomain, DateTime tokenExpirationDate);
  Future<void> writeLastUser(String username, String tenantDomain);
}

class SecureStorageRepositoryImpl implements SecureStorageRepository {
  late final SecureStorageDatasource _datasource;

  SecureStorageRepositoryImpl({
    SecureStorageDatasource? datasource,
  }) {
    _datasource = datasource ?? SecureStorageLocalDatasource();
  }

  @override
  Future<User?> readUser({required String username}) async {
    final userModel = await _datasource.readUser(
      username: username,
    );

    if (userModel != null) {
      return User(
        changePassword: userModel.changePassword,
        admin: userModel.admin,
        allowedToChangePassword: userModel.allowedToChangePassword,
        id: userModel.id,
        username: userModel.username,
        fullName: userModel.fullName,
        email: userModel.email,
        tenantDomain: userModel.tenantDomain,
        tenantName: userModel.tenantName,
        tenantLocale: userModel.tenantLocale,
        blocked: userModel.blocked,
        properties: userModel.properties,
        photoUrl: userModel.photoUrl,
      );
    }

    return null;
  }

  @override
  Future<Token?> readToken({required String username}) async {
    final tokenModel = await _datasource.readToken(
      username: username,
    );

    if (tokenModel != null) {
      return Token(
        version: tokenModel.version,
        expiresIn: tokenModel.expires_in,
        username: tokenModel.username,
        tokenType: tokenModel.token_type,
        accessToken: tokenModel.access_token,
        refreshToken: tokenModel.refresh_token,
        type: tokenModel.type,
        email: tokenModel.email,
        fullName: tokenModel.fullName,
        tenantName: tokenModel.tenantName,
        locale: tokenModel.locale,
        hash: tokenModel.hash,
        salt: tokenModel.salt,
      );
    }

    return null;
  }

  @override
  Future<DateTime?> readTokenExpirationDate({required String username}) async {
    return await _datasource.readTokenExpirationDate(
      username: username,
    );
  }

  @override
  Future<void> writeUser(User user) async {
    final userModel = UserModel(
      changePassword: user.changePassword,
      admin: user.admin,
      allowedToChangePassword: user.allowedToChangePassword,
      id: user.id,
      username: user.username,
      fullName: user.fullName,
      email: user.email,
      tenantDomain: user.tenantDomain,
      tenantName: user.tenantName,
      tenantLocale: user.tenantLocale,
      blocked: user.blocked,
      properties: user.properties,
      photoUrl: user.photoUrl,
    );

    await _datasource.writeUser(userModel);
  }

  @override
  Future<void> writeToken(Token token) async {
    final tokenModel = TokenModel(
      version: token.version,
      expires_in: token.expiresIn,
      username: token.username,
      token_type: token.tokenType,
      access_token: token.accessToken,
      refresh_token: token.refreshToken,
      type: token.type,
      email: token.email,
      fullName: token.fullName,
      tenantName: token.tenantName,
      locale: token.locale,
      hash: token.hash,
      salt: token.salt,
    );

    await _datasource.writeToken(tokenModel);
  }

  @override
  Future<void> writeTokenExpirationDate(String username, String tenantDomain,
      DateTime tokenExpirationDate) async {
    await _datasource.writeTokenExpirationDate(
        username, tenantDomain, tokenExpirationDate);
  }

  @override
  Future<void> deleteUser({required String username}) async {
    await _datasource.deleteUser(username: username);
  }

  @override
  Future<void> deleteToken({required String username}) async {
    await _datasource.deleteToken(username: username);
  }

  @override
  Future<void> deleteTokenExpirationDate({required String username}) async {
    await _datasource.deleteTokenExpirationDate(username: username);
  }

  @override
  Future<String?> readLastUser() async {
    if (SeniorAuthentication.automaticLogon) {
      return await _datasource.readLastUser();
    }
    return null;
  }

  @override
  Future<void> writeLastUser(String username, String tenantDomain) async {
    await _datasource.writeLastUser(username, tenantDomain);
  }

  @override
  Future<void> deleteLastUser() async {
    await _datasource.deleteLastUser();
  }
}
