import 'dart:developer';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../../../../../senior_platform_authentication_ui.dart';

part 'authentication_event.dart';
part 'authentication_state.dart';

/// Authentication business logic component.
///
/// Manages authentication status based on initialize configuration.
/// This instance must be a singleton and must be initialized closest to
/// the root of your application's widget tree.
class AuthenticationBloc
    extends Bloc<AuthenticationEvent, AuthenticationState> {
  final GetUserUsecase _getUserUsecase;
  final StoreAuthenticationDataUsecase _storeAuthenticationDataUsecase;
  final CheckStoredTokenUsecase _checkStoredTokenUsecase;
  final LogoutUsecase _logoutUsecase;
  final ClearStoredDataUsecase _clearStoredDataUsecase;
  final GetStoredTokenUsecase _getStoredTokenUsecase;
  final RefreshStoredTokenUsecase _refreshStoredTokenUsecase;
  final CheckStatusConnectionUsecase _checkStatusConnectionUsecase;

  /// Internal constructor, should not be exposed.
  AuthenticationBloc._internal({
    required GetUserUsecase getUserUsecase,
    required StoreAuthenticationDataUsecase storeAuthenticationDataUsecase,
    required CheckStoredTokenUsecase checkStoredTokenUsecase,
    required LogoutUsecase logoutUsecase,
    required ClearStoredDataUsecase clearStoredDataUsecase,
    required GetStoredTokenUsecase getStoredTokenUsecase,
    required RefreshStoredTokenUsecase refreshStoredTokenUsecase,
    required CheckStatusConnectionUsecase checkStatusConnectionUsecase,
  })  : _getUserUsecase = getUserUsecase,
        _storeAuthenticationDataUsecase = storeAuthenticationDataUsecase,
        _checkStoredTokenUsecase = checkStoredTokenUsecase,
        _logoutUsecase = logoutUsecase,
        _clearStoredDataUsecase = clearStoredDataUsecase,
        _getStoredTokenUsecase = getStoredTokenUsecase,
        _refreshStoredTokenUsecase = refreshStoredTokenUsecase,
        _checkStatusConnectionUsecase = checkStatusConnectionUsecase,
        super(const AuthenticationState.unknown()) {
    on<AuthenticationStatusChanged>(_onAuthenticationStatusChanged);
    on<LogoutOnlineRequested>(_onLogoutOnlineRequested);
    on<CheckAuthenticationRequested>(_onCheckAuthenticationRequested);
    on<LogoutOfflineRequested>(_onLogoutOfflineRequested);
  }

  /// Singleton instance for production environment.
  static final AuthenticationBloc _instance = AuthenticationBloc._internal(
    getUserUsecase: GetUserUsecase(),
    storeAuthenticationDataUsecase: StoreAuthenticationDataUsecase(),
    checkStoredTokenUsecase: CheckStoredTokenUsecase(
      getUserUsecase: GetUserUsecase(),
      checkStatusConnectionUsecase: CheckStatusConnectionUsecase(
          getConnectivityStatusUsecase: GetConnectivityStatusUsecase()),
    ),
    logoutUsecase: LogoutUsecase(),
    clearStoredDataUsecase: ClearStoredDataUsecase(),
    getStoredTokenUsecase: GetStoredTokenUsecase(),
    refreshStoredTokenUsecase: RefreshStoredTokenUsecase(
      getUserUsecase: GetUserUsecase(),
      refreshTokenUsecase: RefreshTokenUsecase(),
      storeAuthenticationDataUsecase: StoreAuthenticationDataUsecase(),
      clearStoredDataUsecase: ClearStoredDataUsecase(),
    ),
    checkStatusConnectionUsecase: CheckStatusConnectionUsecase(
        getConnectivityStatusUsecase: GetConnectivityStatusUsecase()),
  );

  /// Factory exposed constructor.
  factory AuthenticationBloc() => _instance;

  /// Factory mocked exposed constructor. Should only be used for tests.
  factory AuthenticationBloc.mocked({
    required GetUserUsecase getUserUsecase,
    required StoreAuthenticationDataUsecase storeAuthenticationDataUsecase,
    required CheckStoredTokenUsecase checkStoredTokenUsecase,
    required LogoutUsecase logoutOnlineUsecase,
    required ClearStoredDataUsecase clearStoredDataUsecase,
    required GetStoredTokenUsecase getStoredTokenUsecase,
    required RefreshStoredTokenUsecase refreshStoredTokenUsecase,
    required CheckStatusConnectionUsecase checkStatusConnectionUsecase,
  }) {
    return AuthenticationBloc._internal(
      getUserUsecase: getUserUsecase,
      storeAuthenticationDataUsecase: storeAuthenticationDataUsecase,
      checkStoredTokenUsecase: checkStoredTokenUsecase,
      logoutUsecase: logoutOnlineUsecase,
      clearStoredDataUsecase: clearStoredDataUsecase,
      getStoredTokenUsecase: getStoredTokenUsecase,
      refreshStoredTokenUsecase: refreshStoredTokenUsecase,
      checkStatusConnectionUsecase: checkStatusConnectionUsecase,
    );
  }

  Future<void> _onAuthenticationStatusChanged(
    AuthenticationStatusChanged event,
    Emitter<AuthenticationState> emit,
  ) async {
    switch (event.status) {
      case AuthenticationStatus.unauthenticated:
        return emit(const AuthenticationState.unauthenticated());
      case AuthenticationStatus.authenticated:
        final user = await _tryGetUser(
          GetUserInput(
            accessToken: event.authenticationResponse?.token?.accessToken ?? '',
            includePhoto: SeniorAuthentication.includePhoto,
          ),
        );
        final stored = await _storeAuthenticationDataUsecase(
          AuthenticationData(
            user: user,
            token: event.authenticationResponse?.token,
          ),
        );
        return emit(stored
            ? AuthenticationState.authenticated(
                username: event.authenticationResponse?.token?.username)
            : const AuthenticationState.unauthenticated());
      case AuthenticationStatus.unknown:
        return emit(const AuthenticationState.unknown());
      case AuthenticationStatus.offline:
        return emit(AuthenticationState.offline(
            event.authenticationResponse?.token?.username ?? ''));
    }
  }

  void _onLogoutOnlineRequested(
    LogoutOnlineRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    const username = UserName();
    try {
      final isOnline = await _checkStatusConnectionUsecase(NoParams());

      if (isOnline) {
        final token = await _getStoredTokenUsecase(username);
        if (token != null) {
          await _logoutUsecase(token.accessToken);
        }
      }
    } catch (err) {
      log(err.toString());
    } finally {
      await _clearStoredDataUsecase(username);
      emit(const AuthenticationState.unauthenticated());
    }
  }

  void _onLogoutOfflineRequested(
    LogoutOfflineRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    final username = UserName(currentUsername: event.username);
    final token = await _getStoredTokenUsecase(username);
    if (event.eraseToken && token != null) {
      await _clearStoredDataUsecase(username);
    } else {
      //delete only last user stored
      await _clearStoredDataUsecase(const UserName(currentUsername: ''));
    }

    emit(const AuthenticationState.unauthenticated());
  }

  void _onCheckAuthenticationRequested(
    CheckAuthenticationRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    final username = UserName(currentUsername: event.username);
    final isTokenValid = await _checkStoredTokenUsecase(username);

    if (!isTokenValid) {
      final token = await _refreshStoredTokenUsecase(username);

      return emit(token != null
          ? AuthenticationState.authenticated(username: event.username)
          : const AuthenticationState.unauthenticated());
    }

    return emit(AuthenticationState.authenticated(username: event.username));
  }

  Future<User?> _tryGetUser(GetUserInput getUserInput) async {
    try {
      final user = await _getUserUsecase(getUserInput);
      return user;
    } catch (_) {
      return null;
    }
  }
}
