import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:senior_design_system/senior_design_system.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../../../senior_platform_authentication_ui.dart';
import '../../../core/l10n/l10n_extension.dart';
import '../../mfa_authentication/mfa_authentication_screen.dart';
import '../../reset_password/reset_password_screen.dart';
import '../../saml_authentication/saml_authentication_screen.dart';
import 'username_authentication_form.dart';

class UserNameAuthenticationContent extends StatelessWidget {
  final String? helpImageUrl;
  final String? helpImageAsset;
  const UserNameAuthenticationContent({
    super.key,
    this.helpImageUrl = '',
    this.helpImageAsset = '',
  });

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(
        left: SeniorSpacing.normal,
        top: SeniorSpacing.normal,
        right: SeniorSpacing.normal,
      ),
      child: BlocListener<UserNameAuthenticationCubit,
          UserNameAuthenticationState>(
        listenWhen: (previous, current) =>
            (previous.errorType != current.errorType) ||
            (previous.authenticationFlow != current.authenticationFlow),
        listener: (context, state) {
          if (state.errorType != null && state.status == NetworkStatus.idle) {
            ScaffoldMessenger.of(context)
              ..hideCurrentSnackBar()
              ..showSnackBar(
                SeniorSnackBar.error(
                  message: _getSnackbarErrorMessage(context, state.errorType!),
                  action: SeniorSnackBarAction(
                    label: context.l10n.ok,
                    onPressed: () =>
                        ScaffoldMessenger.of(context).hideCurrentSnackBar(),
                  ),
                ),
              );
          }

          if (state.authenticationFlow == AuthenticationFlow.saml) {
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (_) => SAMLAuthenticationScreen(
                  username: state.username,
                  tenantDomain: state.tenantLoginSettings?.tenantDomain ?? '',
                ),
              ),
            );
          }

          if (state.authenticationFlow == AuthenticationFlow.mfa) {
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (_) => MFAAuthenticationScreen(
                  mfaInfo: state.mfaInfo!,
                  helpImageAsset: helpImageAsset,
                  helpImageUrl: helpImageUrl,
                ),
              ),
            );
          }

          if (state.authenticationFlow == AuthenticationFlow.resetPassword &&
              state.authenticationResponse?.resetPasswordInfo != null) {
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (_) => ResetPasswordScreen(
                  resetPasswordInfo:
                      state.authenticationResponse!.resetPasswordInfo!,
                  username: state.username,
                ),
              ),
            );
          }
        },
        child: const CustomScrollView(
          slivers: [
            SliverFillRemaining(
              hasScrollBody: false,
              child: UserNameAuthenticationForm(),
            ),
          ],
        ),
      ),
    );
  }

  String _getSnackbarErrorMessage(BuildContext context, ErrorType errorType) {
    if (errorType == ErrorType.tenantNotFound) {
      return context.l10n.tenantNotFoundErrorMessage;
    }

    if (errorType == ErrorType.unauthorized) {
      return context.l10n.unauthorizedErrorMessage;
    }

    return context.l10n.genericErrorMessage;
  }
}
