import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/datasources/secure_storage_local_datasource.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';

import '../../../../../mocks/token_mock.dart';
import '../../../../../mocks/user_mock.dart';

class MockSecureStorageDatasource extends Mock
    implements SecureStorageDatasource {}

void main() {
  late final SecureStorageDatasource datasource;
  late final SecureStorageRepository repository;
  const String username = 'teste@senior.com.br';
  String tenantDomain = 'senior.com.br';

  setUpAll(() {
    datasource = MockSecureStorageDatasource();

    repository = SecureStorageRepositoryImpl(
      datasource: datasource,
    );

    SeniorAuthentication.initialize();

    registerFallbackValue(userModelMock);
    registerFallbackValue(tokenModelMock);
  });

  group('readUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.readUser(username: username),
      ).thenAnswer((_) async {
        return userModelMock;
      });

      verifyNever(
        () => datasource.readUser(username: username),
      );

      // Act
      final result = await repository.readUser(username: username);

      // Assert
      verify(
        () => datasource.readUser(username: username),
      ).called(1);

      expect(result, isA<User>());
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readUser(username: username),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readUser(username: username),
      );

      // Act
      final action = repository.readUser(username: username);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readUser(username: username),
      ).called(1);
    });
  });

  group('writeUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeUser(any()),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeUser(any()),
      );

      // Act
      await repository.writeUser(userMock);

      // Assert
      verify(
        () => datasource.writeUser(any()),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeUser(any()),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeUser(any()),
      );

      // Act
      final action = repository.writeUser(userMock);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeUser(any()),
      ).called(1);
    });
  });

  group('readToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.readToken(username: username),
      ).thenAnswer((_) async {
        return tokenModelMock;
      });

      verifyNever(
        () => datasource.readToken(username: username),
      );

      // Act
      final result = await repository.readToken(username: username);

      // Assert
      verify(
        () => datasource.readToken(username: username),
      ).called(1);

      expect(result, isA<Token?>());
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readToken(username: username),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readToken(username: username),
      );

      // Act
      final action = repository.readToken(username: username);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readToken(username: username),
      ).called(1);
    });
  });

  group('readTokenExpirationDate', () {
    DateTime tTokenExpirationDate = DateTime.now();
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.readTokenExpirationDate(username: username),
      ).thenAnswer((_) async {
        return tTokenExpirationDate;
      });

      verifyNever(
        () => datasource.readTokenExpirationDate(username: username),
      );

      // Act
      final result =
          await repository.readTokenExpirationDate(username: username);

      // Assert
      verify(
        () => datasource.readTokenExpirationDate(username: username),
      ).called(1);

      expect(result, isA<DateTime?>());
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readTokenExpirationDate(username: username),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readTokenExpirationDate(username: username),
      );

      // Act
      final action = repository.readTokenExpirationDate(username: username);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readTokenExpirationDate(username: username),
      ).called(1);
    });
  });

  group('readLastUser', () {
    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readLastUser(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readLastUser(),
      );

      // Act
      final action = repository.readLastUser();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readLastUser(),
      ).called(1);
    });
  });

  group('writeToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeToken(any()),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeToken(any()),
      );

      // Act
      await repository.writeToken(tokenMock);

      // Assert
      verify(
        () => datasource.writeToken(any()),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeToken(any()),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeToken(any()),
      );

      // Act
      final action = repository.writeToken(tokenMock);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeToken(any()),
      ).called(1);
    });
  });

  group('writeTokenExpirationDate', () {
    DateTime tTokenExpirationDate = DateTime.now();
    String username = 'xpto';
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeTokenExpirationDate(any(), any(), any()),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeTokenExpirationDate(any(), any(), any()),
      );

      // Act
      await repository.writeTokenExpirationDate(
          username, tenantDomain, tTokenExpirationDate);

      // Assert
      verify(
        () => datasource.writeTokenExpirationDate(any(), any(), any()),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeTokenExpirationDate(any(), any(), any()),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeTokenExpirationDate(any(), any(), any()),
      );

      // Act
      final action = repository.writeTokenExpirationDate(
          username, tenantDomain, tTokenExpirationDate);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeTokenExpirationDate(any(), any(), any()),
      ).called(1);
    });
  });

  group('writeLastUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeLastUser(username, tenantDomain),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeLastUser(username, tenantDomain),
      );

      // Act
      await repository.writeLastUser(username, tenantDomain);

      // Assert
      verify(
        () => datasource.writeLastUser(username, tenantDomain),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeLastUser(username, tenantDomain),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeLastUser(username, tenantDomain),
      );

      // Act
      final action = repository.writeLastUser(username, tenantDomain);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeLastUser(username, tenantDomain),
      ).called(1);
    });
  });

  group('deleteUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteUser(username: username),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteUser(username: username),
      );

      // Act
      await repository.deleteUser(username: username);

      // Assert
      verify(
        () => datasource.deleteUser(username: username),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteUser(username: username),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteUser(username: username),
      );

      // Act
      final action = repository.deleteUser(username: username);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteUser(username: username),
      ).called(1);
    });
  });

  group('deleteToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteToken(username: username),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteToken(username: username),
      );

      // Act
      await repository.deleteToken(username: username);

      // Assert
      verify(
        () => datasource.deleteToken(username: username),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteToken(username: username),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteToken(username: username),
      );

      // Act
      final action = repository.deleteToken(username: username);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteToken(username: username),
      ).called(1);
    });
  });

  group('deleteTokenExpirationDate', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteTokenExpirationDate(username: username),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteTokenExpirationDate(username: username),
      );

      // Act
      await repository.deleteTokenExpirationDate(username: username);

      // Assert
      verify(
        () => datasource.deleteTokenExpirationDate(username: username),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteTokenExpirationDate(username: username),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteTokenExpirationDate(username: username),
      );

      // Act
      final action = repository.deleteTokenExpirationDate(username: username);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteTokenExpirationDate(username: username),
      ).called(1);
    });
  });

  group('deleteLastUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteLastUser(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteLastUser(),
      );

      // Act
      await repository.deleteLastUser();

      // Assert
      verify(
        () => datasource.deleteLastUser(),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteLastUser(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteLastUser(),
      );

      // Act
      final action = repository.deleteLastUser();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteLastUser(),
      ).called(1);
    });
  });
}
