import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';

import '../../../../../mocks/token_mock.dart';

class MockSecureStorageRepository extends Mock
    implements SecureStorageRepository {}

void main() {
  late final GetStoredTokenUsecase getStoredTokenUsecase;
  late final SecureStorageRepository repository;
  const tCurrentUsername = 'teste@senior.com.br';
  const tUsername = UserName(currentUsername: tCurrentUsername);

  setUpAll(() {
    repository = MockSecureStorageRepository();
    getStoredTokenUsecase =
        GetStoredTokenUsecase(secureStorageRepository: repository);
  });

  group('getStoredTokenUsecase', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => repository.readToken(username: tCurrentUsername),
      ).thenAnswer((_) async {
        return tokenMock;
      });

      verifyNever(
        () => repository.readToken(username: tCurrentUsername),
      );

      // Act
      final result = await getStoredTokenUsecase(tUsername);

      // Assert
      expect(result, isA<Token?>());

      verify(
        () => repository.readToken(username: tCurrentUsername),
      ).called(1);
    });

    test('should execute successfully with last user', () async {
      // Arrange
      when(
        () => repository.readToken(username: ''),
      ).thenAnswer((_) async {
        return tokenMock;
      });

      when(
        () => repository.readLastUser(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => repository.readToken(username: ''),
      );

      verifyNever(
        () => repository.readLastUser(),
      );

      // Act
      final result = await getStoredTokenUsecase(const UserName());

      // Assert
      expect(result, isA<Token?>());

      verify(
        () => repository.readToken(username: ''),
      ).called(1);

      verify(
        () => repository.readLastUser(),
      ).called(1);
    });

    test('should throw exception when repository throws', () async {
      // Arrange
      when(
        () => repository.readToken(username: tCurrentUsername),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => repository.readToken(username: tCurrentUsername),
      );

      // Act
      final action = getStoredTokenUsecase(tUsername);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => repository.readToken(username: tCurrentUsername),
      ).called(1);
    });
  });
}
