import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';

import '../../../../../mocks/user_mock.dart';

class MockSecureStorageRepository extends Mock
    implements SecureStorageRepository {}

void main() {
  late final GetStoredUserUsecase getStoredUserUsecase;
  late final SecureStorageRepository repository;
  const tCurrentUsername = 'teste@senior.com.br';
  const tUsername = UserName(currentUsername: tCurrentUsername);

  setUpAll(() {
    repository = MockSecureStorageRepository();
    getStoredUserUsecase =
        GetStoredUserUsecase(secureStorageRepository: repository);
  });

  group('getStoredUserUsecase', () {
    test(
        'should execute successfully when '
        'currentUsername is not empty', () async {
      // Arrange
      when(
        () => repository.readUser(username: tCurrentUsername),
      ).thenAnswer((_) async {
        return userMock;
      });

      when(
        () => repository.readLastUser(),
      ).thenAnswer((_) async {
        return tCurrentUsername;
      });

      verifyNever(
        () => repository.readToken(username: tCurrentUsername),
      );

      verifyNever(
        () => repository.readTokenExpirationDate(username: tCurrentUsername),
      );

      // Act
      final result = await getStoredUserUsecase(tUsername);

      // Assert
      expect(result, isA<User?>());

      verify(
        () => repository.readUser(username: tCurrentUsername),
      ).called(1);

      verifyNever(
        () => repository.readLastUser(),
      );
    });

    test(
        'should execute successfully when '
        'currentUsername is empty', () async {
      // Arrange
      when(
        () => repository.readUser(username: tCurrentUsername),
      ).thenAnswer((_) async {
        return userMock;
      });

      when(
        () => repository.readLastUser(),
      ).thenAnswer((_) async {
        return tCurrentUsername;
      });

      verifyNever(
        () => repository.readToken(username: tCurrentUsername),
      );

      verifyNever(
        () => repository.readTokenExpirationDate(username: tCurrentUsername),
      );

      // Act
      final result = await getStoredUserUsecase(const UserName());

      // Assert
      expect(result, isA<User?>());

      verify(
        () => repository.readUser(username: tCurrentUsername),
      ).called(1);

      verify(
        () => repository.readLastUser(),
      ).called(1);
    });

    test('should throw exception when repository throws', () async {
      // Arrange
      when(
        () => repository.readLastUser(),
      ).thenAnswer((_) async {
        return tCurrentUsername;
      });

      when(
        () => repository.readUser(username: tCurrentUsername),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => repository.readUser(username: tCurrentUsername),
      );

      // Act
      final action = getStoredUserUsecase(tUsername);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => repository.readUser(username: tCurrentUsername),
      ).called(1);

      verifyNever(
        () => repository.readLastUser(),
      );
    });
  });
}
