// ignore_for_file: prefer_const_constructors

import 'package:bloc_test/bloc_test.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';

import '../../../../../mocks/authentication_data_mock.dart';
import '../../../../../mocks/get_user_input_mock.dart';
import '../../../../../mocks/token_mock.dart';
import '../../../../../mocks/user_mock.dart';

class MockGetUserUsecase extends Mock implements GetUserUsecase {}

class MockStoreAuthenticationDataUsecase extends Mock
    implements StoreAuthenticationDataUsecase {}

class MockCheckStoredTokenUsecase extends Mock
    implements CheckStoredTokenUsecase {}

class MockLogoutOnlineUsecase extends Mock implements LogoutUsecase {}

class MockClearStoredDataUsecase extends Mock
    implements ClearStoredDataUsecase {}

class MockGetStoredTokenUsecase extends Mock implements GetStoredTokenUsecase {}

class MockRefreshStoredTokenUsecase extends Mock
    implements RefreshStoredTokenUsecase {}

class MockCheckStatusConnectionUsecase extends Mock
    implements CheckStatusConnectionUsecase {}

void main() {
  late GetUserUsecase getUserUsecase;
  late StoreAuthenticationDataUsecase storeAuthenticationDataUsecase;
  late CheckStoredTokenUsecase checkStoredTokenUsecase;
  late LogoutUsecase logoutOnlineUsecase;
  late ClearStoredDataUsecase clearStoredDataUsecase;
  late GetStoredTokenUsecase getStoredTokenUsecase;
  late AuthenticationBloc authenticationBloc;
  late RefreshStoredTokenUsecase refreshStoredTokenUsecase;
  late CheckStatusConnectionUsecase checkStatusConnectionUsecase;
  const String tUsername = 'username';

  setUpAll(() {
    SeniorAuthentication.initialize();
  });

  setUp(() {
    getUserUsecase = MockGetUserUsecase();
    storeAuthenticationDataUsecase = MockStoreAuthenticationDataUsecase();
    checkStoredTokenUsecase = MockCheckStoredTokenUsecase();
    logoutOnlineUsecase = MockLogoutOnlineUsecase();
    getStoredTokenUsecase = MockGetStoredTokenUsecase();
    clearStoredDataUsecase = MockClearStoredDataUsecase();
    refreshStoredTokenUsecase = MockRefreshStoredTokenUsecase();
    checkStatusConnectionUsecase = MockCheckStatusConnectionUsecase();
    authenticationBloc = AuthenticationBloc.mocked(
      getUserUsecase: getUserUsecase,
      storeAuthenticationDataUsecase: storeAuthenticationDataUsecase,
      checkStoredTokenUsecase: checkStoredTokenUsecase,
      logoutOnlineUsecase: logoutOnlineUsecase,
      clearStoredDataUsecase: clearStoredDataUsecase,
      getStoredTokenUsecase: getStoredTokenUsecase,
      refreshStoredTokenUsecase: refreshStoredTokenUsecase,
      checkStatusConnectionUsecase: checkStatusConnectionUsecase,
    );
    registerFallbackValue(authenticationDataMock);
    registerFallbackValue(NoParams());
    registerFallbackValue(UserName());
    registerFallbackValue(getUserInputMock);
  });

  group('factory', () {
    test('AuthenticationBloc factory', () {
      final leafAuthenticationBloc = AuthenticationBloc();
      expect(leafAuthenticationBloc, isA<AuthenticationBloc>());
    });
  });

  group('onAuthenticationStatusChanged', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unknown] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.unknown',
      build: () => authenticationBloc,
      act: (bloc) => bloc
          .add(const AuthenticationStatusChanged(AuthenticationStatus.unknown)),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unknown(),
      ],
      verify: (_) async {
        verifyNever(() => getUserUsecase.call(any()));
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.unauthenticated',
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(const AuthenticationStatusChanged(
          AuthenticationStatus.unauthenticated)),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verifyNever(() => getUserUsecase.call(any()));
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.authenticated] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.authenticated '
      'and verify usecases call',
      setUp: () {
        when(() => getUserUsecase.call(any()))
            .thenAnswer((_) async => userMock);
        when(() => storeAuthenticationDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(const AuthenticationStatusChanged(
        AuthenticationStatus.authenticated,
        authenticationResponse: authenticationResponseMock,
      )),
      expect: () => <AuthenticationState>[
        AuthenticationState.authenticated(username: tUsername),
      ],
      verify: (_) async {
        verify(() => getUserUsecase.call(any())).called(1);
        verify(() => storeAuthenticationDataUsecase.call(any())).called(1);
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.offline',
      setUp: () {
        when(() => checkStatusConnectionUsecase.call(NoParams())).thenAnswer(
          (_) async => false,
        );
        when(() => getUserUsecase.call(any()))
            .thenAnswer((_) async => userMock);
        when(() => storeAuthenticationDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(const AuthenticationStatusChanged(
        AuthenticationStatus.offline,
        authenticationResponse: authenticationResponseMock,
      )),
      expect: () => <AuthenticationState>[
        AuthenticationState.offline(tUsername),
      ],
      verify: (_) async {
        verifyNever(() => getUserUsecase.call(any()));
      },
    );
  });

  group('onLogoutOnlineRequested', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when LogoutOnlineRequested '
      'event if fired and verify usecases call',
      setUp: () {
        when(() => checkStatusConnectionUsecase.call(NoParams())).thenAnswer(
          (_) async => true,
        );
        when(() => getStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => tokenMock);
        when(() => logoutOnlineUsecase.call(any()))
            .thenAnswer((_) async => true);
        when(() => clearStoredDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(LogoutOnlineRequested()),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => getStoredTokenUsecase.call(any())).called(1);
        verify(() => logoutOnlineUsecase.call(any())).called(1);
        verify(() => clearStoredDataUsecase.call(any())).called(1);
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when LogoutOnlineRequested '
      'event if fired fail and verify usecases call',
      setUp: () {
        when(() => checkStatusConnectionUsecase.call(NoParams())).thenAnswer(
          (_) async => true,
        );
        when(() => getStoredTokenUsecase.call(any()))
            .thenThrow((_) async => Exception());
        when(() => logoutOnlineUsecase.call(any()))
            .thenAnswer((_) async => false);
        when(() => clearStoredDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(LogoutOnlineRequested()),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => getStoredTokenUsecase.call(any())).called(1);
        verifyNever(() => logoutOnlineUsecase.call(any()));
        verify(() => clearStoredDataUsecase.call(any())).called(1);
      },
    );
  });

  group('onCheckAuthenticationRequested', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.authenticated] when CheckAuthenticationRequested '
      'event if fired with valid token on storage and verify usecases call',
      setUp: () {
        when(() => checkStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) =>
          bloc.add(CheckAuthenticationRequested(username: tUsername)),
      expect: () => <AuthenticationState>[
        AuthenticationState.authenticated(username: tUsername),
      ],
      verify: (_) async {
        verify(() => checkStoredTokenUsecase.call(any())).called(1);
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when CheckAuthenticationRequested '
      'event if fired with invalid token on storage and verify usecases call',
      setUp: () {
        when(() => checkStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => false);
        when(() => refreshStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => null);
      },
      build: () => authenticationBloc,
      act: (bloc) =>
          bloc.add(CheckAuthenticationRequested(username: tUsername)),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => checkStoredTokenUsecase.call(any())).called(1);
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.authenticated] when CheckAuthenticationRequested '
      'event if fired with invalid token on storage and refreshToken usecases call',
      setUp: () {
        when(() => checkStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => false);
        when(() => refreshStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => tokenMock);
      },
      build: () => authenticationBloc,
      act: (bloc) =>
          bloc.add(CheckAuthenticationRequested(username: tUsername)),
      expect: () => <AuthenticationState>[
        AuthenticationState.authenticated(username: tUsername),
      ],
      verify: (_) async {
        verify(() => checkStoredTokenUsecase.call(any())).called(1);
        verify(() => refreshStoredTokenUsecase.call(any())).called(1);
      },
    );
  });

  group('onLogoutOfflineRequested', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when LogoutOfflineRequested default '
      'event if fired and verify usecases call ',
      setUp: () {
        when(() => getStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => tokenMock);
        when(() => clearStoredDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(LogoutOfflineRequested(username: tUsername)),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => getStoredTokenUsecase.call(any())).called(1);
        verify(() => clearStoredDataUsecase.call(any())).called(1);
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when LogoutOfflineRequested erase token'
      'event if fired and verify usecases call',
      setUp: () {
        when(() => getStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => tokenMock);
        when(() => clearStoredDataUsecase.call(any()))
            .thenAnswer((_) async => false);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(LogoutOfflineRequested(
        username: tUsername,
        eraseToken: true,
      )),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => getStoredTokenUsecase.call(any())).called(1);
        verify(() => clearStoredDataUsecase.call(any())).called(1);
      },
    );
  });
}
